Package: SynMut
Type: Package
Title: SynMut: Designing Synonymously Mutated Sequences with Different
        Genomic Signatures
Version: 1.27.0
Authors@R: c(person("Haogao", "Gu", email = "hggu@connect.hku.hk", role = c("aut", "cre")),
    person("Leo L.M.", "Poon", email = "llmpoon@hku.hk", role = "led"))
Description: 
    There are increasing demands on designing virus mutants with specific dinucleotide or codon composition. 
    This tool can take both dinucleotide preference and/or codon usage bias into account while designing mutants.
    It is a powerful tool for in silico designs of DNA sequence mutants.
License: GPL-2
Encoding: UTF-8
Suggests: BiocManager, knitr, rmarkdown, testthat, devtools, prettydoc,
        glue
VignetteBuilder: knitr
Imports: seqinr, methods, Biostrings, stringr, BiocGenerics
RoxygenNote: 7.1.0
Collate: 'regioned_dna_Class.R' 'codon_mimic.R' 'input_seq.R'
        'codon_random.R' 'codon_to.R' 'dinu_to.R' 'distance_analysis.R'
        'region_related.R' 'seq_random.R' 'zzz.R'
biocViews: SequenceMatching, ExperimentalDesign, Preprocessing
BugReports: https://github.com/Koohoko/SynMut/issues
URL: https://github.com/Koohoko/SynMut
git_url: https://git.bioconductor.org/packages/SynMut
git_branch: devel
git_last_commit: 0bb1b93
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-11-01 00:33:18 UTC; biocbuild
Author: Haogao Gu [aut, cre],
  Leo L.M. Poon [led]
Maintainer: Haogao Gu <hggu@connect.hku.hk>
